import {
    CollectionViewer,
    DataSource
} from '@angular/cdk/collections';

import { Observable } from 'rxjs/Observable';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { of } from 'rxjs/observable/of';

import {
    catchError,
    finalize
} from 'rxjs/operators';

import { CommentsService } from './comments.service';

import { Comment } from '../models/comment';


export class CommentsDataSource implements DataSource<Comment> {
    private commentsSubject = new BehaviorSubject<Comment[]>([]);
    private countSubject = new BehaviorSubject<number>(0);
    private loadingSubject = new BehaviorSubject<boolean>(false);

    public count$ = this.countSubject.asObservable();
    public loading$ = this.loadingSubject.asObservable();

    constructor(private commentsService: CommentsService) { }

    loadComments(videoID: string,
                 filter: string,
                 sortColumn: string,
                 sortDirection: string,
                 pageIndex: number,
                 pageSize: number) {

        this.loadingSubject.next(true);

        this.commentsService.findComments(videoID, filter, sortColumn, sortDirection,
                                          pageIndex, pageSize)
            .pipe(
                catchError(() => of([])),
                finalize(() => this.loadingSubject.next(false))
            )
            .subscribe((data: any) => {
                this.commentsSubject.next(data.comments);
                this.countSubject.next(data.totalCount);
            });
    }

    connect(collectionViewer: CollectionViewer): Observable<Comment[]> {
        console.log('Connecting data source');
        return this.commentsSubject.asObservable();
    }

    disconnect(collectionViewer: CollectionViewer): void {
        this.commentsSubject.complete();
        this.countSubject.complete();
        this.loadingSubject.complete();
    }
}
